"""Site

Stone's representation of a website
"""
# -*- coding: utf-8 -*-

from collections import UserDict
from json import JSONEncoder
import os
from typing import Dict, List
from markdown import Markdown

from stone.page import Page, PageEncoder
from stone.resource import Resource


class SiteEncoder(JSONEncoder):
    """JSON encoder for Site"""

    def default(self, o):
        if isinstance(o, Site):
            items = ['site', 'type', 'source', 'target', 'templates']
            result = {}
            for item in items:
                try:
                    result[item] = o[item]
                except KeyError:
                    pass
            result['pages'] = o.pages
            return result
        if isinstance(o, Page):
            return PageEncoder().default(o)

        # When not a page, call the JSONEncoder. It will call the correct fail.
        return JSONEncoder.default(self, o)


class Site(UserDict):  # pylint: disable=too-many-ancestors
    """Representation of a Site"""

    root: str
    pages: List[Page]
    templates: List[str]
    resources: List[str]
    data: Dict[str, str]

    def __init__(self, root: str, data: Dict) -> None:
        super().__init__()
        self.root = root
        self.data = data
        self.pages = []
        self.templates = []
        self.resources = []
        self._parse(data)

    def __contains__(self, key):
        return str(key) in self.data

    def __repr__(self):
        return "Site(%r, %r)" % (self.root, self.data)

    def __str__(self):
        """Return the stringified version of Site

        Return a selective dictionary of Site as a string
        """
        items = ['site', 'type', 'source', 'target', 'templates']
        result = {}
        for item in items:
            try:
                result[item] = self[item]
            except KeyError:
                pass
        result['pages'] = self.pages
        return str(result)

    def __setitem__(self, key, item):
        self.data[str(key)] = item

    def _parse(self, data):
        """Load pages to be generated"""
        try:
            for page_data in data['pages']:
                self.pages.append(
                    Page(
                        self,
                        page_data.pop('source'),
                        page_data.pop('target'),
                        data=page_data))

            self.templates = [
                os.path.join(self.root, template)
                for template in data["templates"]
            ]
            self.resources = [
                Resource(self.root, **resource)
                for resource in data["resources"]
            ]
        except KeyError as key_error:
            if key_error == 'templates':
                print("No temaplates found for %s" % (data["site"]))

    def add_page(self, page):
        """Add supplied page to site"""
        self.pages.append(page)

    def is_blog(self):
        """Returns if the site is a blog or not"""
        try:
            return self.data['type'] == 'blog'
        except KeyError:
            return False

    def render(self, environment):
        """Render Markdown to HTML and extract YAML metadata"""
        renderer = None
        for page in self.pages:
            # Pages require initial parsing to read their YAML metadata
            renderer = Markdown(extensions=[
                'markdown.extensions.meta', 'markdown.extensions.tables',
                'markdown.extensions.footnotes'
            ])
            page.convert_to_template_html(renderer)
        for page in self.pages:
            if 'page_type' in page and page['page_type'] == "index":
                """
                Pass all blog posts to the index page, do not pass other
                indexes or page types to the index.
                """
                page['posts'] = [
                    post for post in self.pages if post is not page
                ]
                page['posts'].reverse()
            print("Rendering: {} to {}".format(page['source_path'],
                                               page['target_path']))
            page.render_html(environment)

        for resource in self.resources:
            resource.render()



"""Config

Stone's representation for site.json
"""
import errno
import json
import os
import sys

from stone.site import Site, SiteEncoder


class Config(object):
    """Loader site.json"""

    site_config_file = "site.json"

    def __init__(self):
        pass

    def read(self, path):
        """Load site configuration"""
        configs = []
        try:
            json_data = json.loads(
                open(os.path.join(path, self.site_config_file), "r").read())
        except FileNotFoundError as fnf:
            if fnf.errno != errno.ENOENT:
                raise
            else:
                print("[ERROR] No path to site config")
                return 1

        try:
            if json_data["version"] != 1:
                print(
                    "This is an older site.json and it doesn't checkout",
                    file=sys.stderr)
                exit(1)
            for site_data in json_data["sites"]:
                configs.append(Site(path, site_data))
        except KeyError:
            configs.append(Site(path, site_data))

        return configs

    def write(self, path, sites):
        """Serialize a site to JSON"""
        config = {'version': 1}
        config['sites'] = sites
        with open(os.path.join(path, self.site_config_file), "w") as cfg_file:
            cfg_file.write(json.dumps(config, cls=SiteEncoder, indent=4))



"""Setup script for Stone

See: https://github.com/NeuralSandwich/stone
"""

# To use a consistent encoding
# pylint: disable=redefined-builtin
from codecs import open
from os import path

# Apparently you should always prefer setuptools over distutils
from setuptools import setup, find_packages

# pylint: disable=invalid-name
here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='stone-site',

      # Versions should comply with PEP440. For a discussion on single-sourcing
      version='0.1a5',


      description='Static site generator',
      long_description=long_description,

      # The project's main homepage
      url='https://github.com/NeuralSandwich/stone',

      # Author's details
      author='Sean Jones',
      author_email='sean@half.systems',

      license='MIT',

      # https://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          # How mature is this project? Common values are
          #   3 - Alpha
          #   4 - Beta
          #   5 - Production/Stable
          'Development Status :: 3 - Alpha',

          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          ],

      # What does this project relate to?
      keywords='web html markdown static content',
      packages=find_packages(exclude=['contrib', 'docs', 'tests']),

      # List run-time dependencies here.
      # https://packaging.python.org/en/latest/requirements.html
      install_requires=['css_html_js_minify', 'jinja2', 'Markdown',
                        'unidecode'],
      extras_require={
          'dev': ['check-manifest'],
          'test': ['coverage'],
      },
      entry_points={
          'console_scripts': ['stone=stone.__main__:main']
      }
     )



"""Entry point for Stone"""

from __future__ import print_function
import argparse
import os

from stone.stone import generate_site, init_site, new_page


def add_newpage(parser):
    """Add arguments for newpage command"""
    subparser = parser.add_parser(
        'newpage', help=('add a new page to  site.json and an emtpy file'))
    subparser.add_argument(
        "source",
        type=str,
        help='input filename')
    subparser.add_argument(
        "--target",
        type=str,
        help='output filename')
    subparser.add_argument(
        "--page-type",
        default="post",
        type=str,
        help='type of page to generate')
    subparser.set_defaults(func=new_page)


def add_init(parser):
    """Add arguments for init command"""
    subparser = parser.add_parser(
        'init', help=('create a template site.json'))
    subparser.add_argument(
        "--type", default="blog", type=str, help='type of site to generate')
    subparser.add_argument(
        "--site-name",
        type=str,
        help='name of the site: example.com',
        required=True)
    subparser.set_defaults(func=init_site)


def add_generate(parser):
    """Add arguments for generate command"""
    subparser = parser.add_parser(
        'generate', aliases=['gen', 'build'], help=('generate site'))
    subparser.set_defaults(func=generate_site)


def main(args=None):
    """Entry point function for Stone"""
    parser = argparse.ArgumentParser(
        prog='stone', description='Static website generator')

    # Add general arguments after subparsers so the order makes sense
    parser.add_argument("site_root", help='website root directory')

    subparsers = parser.add_subparsers(title='commands', help='commands')

    # stone build <path>
    add_generate(subparsers)

    # stone init <path>
    add_init(subparsers)

    # stone newpage <path>
    add_newpage(subparsers)
    args = parser.parse_args()

    if not os.path.isdir(args.site_root):
        print("[ERROR] %s is not a directory" % args.site_root)
        parser.print_help()
        return 1

    args.func(args)


if __name__ == '__main__':
    main()
